@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2018 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -54,7 +54,7 @@ public:
   derived_probe* current_probe;
   symresolution_info (systemtap_session& s, bool omniscient_unmangled = false);
 
-  vardecl* find_var (interned_string name, int arity, const token *tok);
+  vardecl* find_var (const std::string& name, int arity, const token *tok);
   std::vector<functiondecl*> find_functions (functioncall *call, const std::string& name, unsigned arity, const token *tok);
   std::set<std::string> collect_functions(void);
 
@@ -64,6 +64,8 @@ public:
   void visit_arrayindex (arrayindex* e);
   void visit_arrayindex (arrayindex *e, bool wildcard_ok);
   void visit_functioncall (functioncall* e);
+  void visit_embeddedcode (embeddedcode* s);  
+  void visit_embedded_expr (embedded_expr* e);
   void visit_delete_statement (delete_statement* s);
   void visit_array_in (array_in *e);
 };
@@ -121,6 +123,10 @@ struct typeresolution_info: public visitor
   // If they have this null_type, they're explicitly *not* a rich type.
   const exp_type_ptr null_type;
 
+  // PR24199 NB: below functions should NOT throw exceptions on a
+  // routine type resolution failure.  Instead, session.print_error()
+  // and keep on chugging.
+
   void visit_block (block* s);
   void visit_try_block (try_block* s);
   void visit_embeddedcode (embeddedcode* s);
@@ -166,6 +172,10 @@ struct typeresolution_info: public visitor
   void visit_defined_op (defined_op* e);
   void visit_entry_op (entry_op* e);
   void visit_perf_op (perf_op* e);
+
+  // PR24199 NB: above functions should NOT throw exceptions on a
+  // routine type resolution failure.  Instead, session.print_error()
+  // and keep on chugging.
 };
 
 
@@ -189,7 +199,7 @@ struct derived_probe: public probe
   virtual void join_group (systemtap_session& s) = 0;
   virtual probe_point* sole_location () const;
   virtual probe_point* script_location () const;
-  virtual void printsig (std::ostream &o) const;
+  virtual void printsig (std::ostream &o, bool nest=true) const;
   // return arguments of probe if there
   virtual void getargs (std::list<std::string> &) const {}
   void printsig_nested (std::ostream &o) const;
