@@ -14,13 +14,11 @@ PIDDIR=/run/systemtap
 mkdir -p $PIDDIR
 
 for script in $ONBOOT_SCRIPTS; do
-   pid=$PIDDIR/$script
    eval opts=\$${script}_OPT
    if [ $LOG_BOOT_ERR -eq 1 ]; then
       $STAPRUN $opts $CACHE_PATH/$script.ko 2> $PIDDIR/$script.log
    else
       $STAPRUN $opts $CACHE_PATH/$script.ko
    fi
-   echo 0 > $pid
 done
 
