@@ -22,7 +22,6 @@ using namespace std;
 
 extern "C" {
 
-#define _RPM_4_4_COMPAT
 #include <string.h>
 #include <rpm/rpmlib.h>
 #include <rpm/rpmts.h>
@@ -100,8 +99,7 @@ missing_rpm_enlist (systemtap_session& sess, const char *filename, const char *r
 	    break;
 	  /* Verify the kernel file is not already installed.  */
 
-	  rpminfo = headerSprintf(h, header,
-			      rpmTagTable, rpmHeaderFormats, &err);
+	  rpminfo = headerFormat(h, header, &err);
 
 	  if (!rpminfo)
 	    {
@@ -154,9 +152,8 @@ missing_rpm_enlist (systemtap_session& sess, const char *filename, const char *r
 	  /* The allocated memory gets utilized below for MISSING_RPM_HASH.  */
           if(strcmp(rpm_type,"-debuginfo")==0){
 	    xfree(rpminfo);
-	    rpminfo = headerSprintf(h,
-		      "%{name}-%{version}-%{release}.%{arch}",
-		      rpmTagTable, rpmHeaderFormats, &err);
+	    rpminfo = headerFormat(h, "%{name}-%{version}-%{release}.%{arch}",
+				   &err);
 	  }
 	  if (!rpminfo)
 	    {
