@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Print Functions
- * Copyright (C) 2012 Red Hat Inc.
+ * Copyright (C) 2012-2018 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -21,7 +21,7 @@
 #include "transport.c"
 #include "vsprintf.c"
 
-static void _stp_print_kernel_info(char *vstr, int ctx, int num_probes)
+static void _stp_print_kernel_info(char *sname, char *vstr, int ctx, int num_probes)
 {
 	// nah...
 }
