@@ -1,5 +1,5 @@
 /* main header file for Linux
- * Copyright (C) 2005-2014 Red Hat Inc.
+ * Copyright (C) 2005-2019 Red Hat Inc.
  * Copyright (C) 2005-2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -34,7 +34,7 @@
 #include <linux/compat.h>
 #include <linux/sched.h>
 #include <linux/mm.h>
-#include <linux/timer.h>
+#include "timer_compatibility.h"
 #include <linux/delay.h>
 #include <linux/profile.h>
 #include <linux/rcupdate.h>
@@ -62,9 +62,14 @@
 #define STP_TRANSPORT_VERSION 1
 #endif
 
-#ifdef STAPCONF_UDELAY_SIMPLE
+#if defined(STAPCONF_UDELAY_SIMPLE_EXPORTED)
 #undef udelay
 #define udelay(x) udelay_simple(x)
+#elif defined(STAPCONF_UDELAY_SIMPLE)
+#undef udelay
+static void *kallsyms_udelay_simple;
+typedef typeof(&udelay_simple) udelay_simple_fn;
+#define udelay(x) ((* (udelay_simple_fn)(kallsyms_udelay_simple))((x)))
 #endif
 
 #ifndef clamp
@@ -190,6 +195,8 @@ static void *kallsyms_uprobe_get_swbp_addr;
 /* task_work functions lack the necessary SYMBOL_EXPORT's */
 #if !defined(STAPCONF_TASK_WORK_ADD_EXPORTED)
 static void *kallsyms_task_work_add;
+#endif
+#if !defined(STAPCONF_TASK_WORK_CANCEL_EXPORTED)
 static void *kallsyms_task_work_cancel;
 #endif
 
