@@ -1,6 +1,6 @@
 /*  -*- linux-c -*-
  * Stack tracing functions
- * Copyright (C) 2005-2009, 2014 Red Hat Inc.
+ * Copyright (C) 2005-2009, 2014-2019 Red Hat Inc.
  * Copyright (C) 2005 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -38,12 +38,48 @@
 
 #include "linux/uprobes-inc.h"
 
-#if defined(STAPCONF_KERNEL_STACKTRACE) || defined(STAPCONF_KERNEL_STACKTRACE_NO_BP)
 #include <linux/stacktrace.h>
+
+#if defined(STAPCONF_KERNEL_STACKTRACE) || defined(STAPCONF_KERNEL_STACKTRACE_NO_BP)
 #include <asm/stacktrace.h>
 #endif
 
-static void _stp_stack_print_fallback(unsigned long, int, int, int);
+#if defined(STAPCONF_KERNEL_UNWIND_STACK)
+#include <asm/unwind.h>
+#endif
+
+#if defined(STAPCONF_STACK_TRACE_SAVE_REGS) /* linux 5.2+ apprx. */
+static __typeof__(stack_trace_save_regs) (*stack_trace_save_regs_fn); /* not exported */
+
+static int
+_stp_init_stack(void)
+{
+	stack_trace_save_regs_fn = (void*) kallsyms_lookup_name("stack_trace_save_regs");
+	dbug_unwind(1, "stack_trace_saves_regs_fn=%lx for _stp_stack_print_fallback().\n",
+		    (unsigned long) stack_trace_save_regs_fn);
+	return 0;
+}
+
+#else /* ! STAPCONF_STACK_TRACE_SAVE_REGS */
+
+static void (*(save_stack_trace_regs_fn))(struct pt_regs *regs,
+				  struct stack_trace *trace);
+
+static int
+_stp_init_stack(void)
+{
+	/* check for save_stack_trace_regs function for fallback stack print */
+	save_stack_trace_regs_fn = (void *)kallsyms_lookup_name("save_stack_trace_regs");
+	dbug_unwind(1, "save_stack_trace_regs_fn=%lx for _stp_stack_print_fallback().\n",
+		    (unsigned long) save_stack_trace_regs_fn);
+	return 0;
+}
+
+#endif /* STAPCONF_STACK_TRACE_SAVE_REGS */
+
+
+
+static void _stp_stack_print_fallback(unsigned long, struct pt_regs*, int, int, int);
 
 #ifdef STP_USE_DWARF_UNWINDER
 #ifdef STAPCONF_LINUX_UACCESS_H
@@ -54,24 +90,6 @@ static void _stp_stack_print_fallback(unsigned long, int, int, int);
 #include <linux/types.h>
 #define intptr_t long
 #define uintptr_t unsigned long
-
-static int _stp_valid_pc_addr(unsigned long addr, struct task_struct *tsk)
-{
-	/* Just a simple check of whether the the address can be accessed
-	   as a user space address. Zero is always bad. */
-
-/* FIXME for s390x PR13350. */
-#if defined (__s390__) || defined (__s390x__)
-       return addr != 0L;
-#else
-	int ok;
-	mm_segment_t oldfs = get_fs();
-	set_fs(USER_DS);
-	ok = access_ok(VERIFY_READ, (long *) (intptr_t) addr, sizeof(long));
-	set_fs(oldfs);
-	return addr != 0L && tsk != NULL ? ok : ! ok;
-#endif
-}
 #endif
 
 #if defined (__ia64__)
@@ -146,7 +164,7 @@ static const struct stacktrace_ops print_stack_ops = {
 };
 
 /* Used for kernel backtrace printing when other mechanisms fail. */
-static void _stp_stack_print_fallback(unsigned long stack,
+static void _stp_stack_print_fallback(unsigned long stack, struct pt_regs *regs,
 				      int sym_flags, int levels, int skip)
 {
         struct print_stack_data print_data;
@@ -154,23 +172,69 @@ static void _stp_stack_print_fallback(unsigned long stack,
         print_data.levels = levels;
         print_data.skip = skip;
 #if defined(STAPCONF_KERNEL_STACKTRACE)
+        dbug_unwind(1, "fallback kernel stacktrace\n");
         dump_trace(current, NULL, (long *)stack, 0, &print_stack_ops,
                    &print_data);
 #else
 	/* STAPCONF_KERNEL_STACKTRACE_NO_BP */
+        dbug_unwind(1, "fallback kernel stacktrace (no bp)\n");
         dump_trace(current, NULL, (long *)stack, &print_stack_ops,
                    &print_data);
 #endif
 }
 #else
-static void _stp_stack_print_fallback(unsigned long s, int v, int l, int k) {
-	/* Don't guess, just give up. */
-	_stp_print_addr(0, v | _STP_SYM_INEXACT, NULL);
+static void _stp_stack_print_fallback(unsigned long sp, struct pt_regs *regs,
+				      int sym_flags,
+				      int levels, int skip) {
+	unsigned long entries[MAXBACKTRACE];
+        unsigned i;
+        unsigned num_entries;
+        
+#if defined(STAPCONF_STACK_TRACE_SAVE_REGS) /* linux 5.2+ apprx. */
+	if (!stack_trace_save_regs_fn) {
+		dbug_unwind(1, "no fallback kernel stacktrace (giving up)\n");
+		_stp_print_addr(0, sym_flags | _STP_SYM_INEXACT, NULL);
+		return;
+	}
+
+        num_entries = (*stack_trace_save_regs_fn)(regs, &entries[0], MAXBACKTRACE, skip);
+#else
+	struct stack_trace trace;
+	/* If don't have save_stack_trace_regs unwinder, just give up. */
+	if (!save_stack_trace_regs_fn) {
+		dbug_unwind(1, "no fallback kernel stacktrace (giving up)\n");
+		_stp_print_addr(0, sym_flags | _STP_SYM_INEXACT, NULL);
+		return;
+	}
+
+	/* Use kernel provided save_stack_trace_regs unwinder if available */
+	dbug_unwind(1, "fallback kernel stacktrace (save_stack_trace_regs)\n");
+	memset(&trace, 0, sizeof(trace));
+	trace.max_entries = MAXBACKTRACE;
+	trace.entries = &(entries[0]);
+	trace.skip = skip;
+	(* (save_stack_trace_regs_fn))(regs, &trace);
+
+	dbug_unwind(1, "trace.nr_entries: %d\n", trace.nr_entries);
+	dbug_unwind(1, "trace.max_entries: %d\n", trace.max_entries);
+	dbug_unwind(1, "trace.skip %d\n", trace.skip);
+        num_entries = trace.nr_entries;
+#endif
+
+	/* save_stack_trace_reg() adds a ULONG_MAX after last valid entry. Ignore it. */
+	for (i=0; i<MAXBACKTRACE && i<num_entries && entries[i]!=ULONG_MAX; ++i) {
+		/* When we have frame pointers, the unwind addresses can be
+		   (mostly) trusted, otherwise it is all guesswork.  */
+#ifdef CONFIG_FRAME_POINTER
+		_stp_print_addr((unsigned long) entries[i], sym_flags, NULL);
+#else
+		_stp_print_addr((unsigned long) entries[i], sym_flags | _STP_SYM_INEXACT,
+				NULL);
+#endif
+	}
 }
-
 #endif /* defined(STAPCONF_KERNEL_STACKTRACE) || defined(STAPCONF_KERNEL_STACKTRACE_NO_BP) */
 
-
 /** Gets user space registers when available, also sets context
  * full_uregs_p if appropriate.  Should be used instead of accessing
  * context uregs field directly when (full) uregs are needed from
@@ -264,7 +328,8 @@ static struct pt_regs *_stp_get_uregs(struct context *c)
 }
 
 
-static unsigned long _stp_stack_unwind_one_kernel(struct context *c, unsigned depth)
+static unsigned long
+_stp_stack_unwind_one_kernel(struct context *c, unsigned depth)
 {
 	struct pt_regs *regs = NULL;
 	struct unwind_frame_info *info = NULL;
@@ -327,7 +392,8 @@ static unsigned long _stp_stack_unwind_one_kernel(struct context *c, unsigned de
 		    (unsigned long long) UNW_SP(info));
 
 	/* check if unwind hit an error */
-	if (ret || ! _stp_valid_pc_addr(UNW_PC(info), NULL)) {
+	if (ret || _stp_lookup_bad_addr(VERIFY_READ, sizeof(long),
+					UNW_PC(info), KERNEL_DS)) {
 		return 0;
 	}
 
@@ -398,6 +464,7 @@ static void _stp_stack_kernel_print(struct context *c, int sym_flags)
 		if (l == 0) {
 			remaining = MAXBACKTRACE - n;
 			_stp_stack_print_fallback(UNW_SP(&c->uwcontext_kernel.info),
+						  &c->uwcontext_kernel.info.regs,
 						  sym_flags, remaining, 0);
 			break;
 		} else {
@@ -424,7 +491,7 @@ static void _stp_stack_kernel_print(struct context *c, int sym_flags)
 		sp = 0;
 		skip = 5; /* yes, that many framework frames. */
 #endif
-		_stp_stack_print_fallback(sp, sym_flags,
+		_stp_stack_print_fallback(sp, NULL, sym_flags,
 					  MAXBACKTRACE, skip);
 #else
 		if (sym_flags & _STP_SYM_SYMBOL)
@@ -441,7 +508,8 @@ static void _stp_stack_kernel_print(struct context *c, int sym_flags)
 #endif
 }
 
-static unsigned long _stp_stack_unwind_one_user(struct context *c, unsigned depth)
+static unsigned long
+_stp_stack_unwind_one_user(struct context *c, unsigned depth)
 {
 	struct pt_regs *regs = NULL;
 	int uregs_valid = 0;
@@ -512,7 +580,8 @@ static unsigned long _stp_stack_unwind_one_user(struct context *c, unsigned dept
 		    (unsigned long long) UNW_PC(info), (unsigned long long) UNW_SP(info));
 
 	/* check if unwind hit an error */
-	if (ret || ! _stp_valid_pc_addr(UNW_PC(info), current)) {
+	if (ret || _stp_lookup_bad_addr(VERIFY_READ, sizeof(long),
+					UNW_PC(info), USER_DS)) {
 		return 0;
 	}
 
