@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Symbolic Lookup Functions
- * Copyright (C) 2005-2018 Red Hat Inc.
+ * Copyright (C) 2005-2019 Red Hat Inc.
  * Copyright (C) 2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -636,30 +636,46 @@ unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *tas
   return 0;
 }
 
+
+// Compare two build-id hex strings, each of length m->build_id_len bytes.
+// Since mismatches can mystify, produce a hex-textual version of both
+// expected and actual strings, and compare textually.  Failure messages
+// are more intelligible this way.
 static int _stp_build_id_check (struct _stp_module *m,
 				unsigned long notes_addr,
 				struct task_struct *tsk)
 {
-  int j;
+  enum { max_buildid_hexstring = 65 };
+  static const char hexnibble[16]="0123456789abcdef";
+  char hexstring_theory[max_buildid_hexstring], hexstring_practice[max_buildid_hexstring];
+  int buildid_len = min((max_buildid_hexstring-1)/2, m->build_id_len);
 
-  for (j = 0; j < m->build_id_len; j++) {
+  int i, j;
+  
+  memset(hexstring_theory, '\0', max_buildid_hexstring);
+  for (i=0, j=0; j<buildid_len; j++) {
+     unsigned char theory = m->build_id_bits[j];
+     hexstring_theory[i++] = hexnibble[theory >> 4];
+     hexstring_theory[i++] = hexnibble[theory & 15];
+  }
+
+  memset(hexstring_practice, '\0', max_buildid_hexstring);
+  for (i=0, j=0; j<buildid_len; j++) {
     /* Use set_fs / get_user to access conceivably invalid addresses.
      * If loc2c-runtime.h were more easily usable, a deref() loop
      * could do it too. */
     mm_segment_t oldfs = get_fs();
     int rc;
-    unsigned char theory, practice = 0;
+    unsigned char practice = 0;
 
 #ifdef STAPCONF_PROBE_KERNEL
     if (!tsk) {
-      theory = m->build_id_bits[j];
       set_fs(KERNEL_DS);
       rc = probe_kernel_read(&practice, (void*)(notes_addr + j), 1);
     }
     else
 #endif
     {
-      theory = m->build_id_bits[j];
       set_fs (tsk ? USER_DS : KERNEL_DS);
 
       /*
@@ -685,21 +701,19 @@ static int _stp_build_id_check (struct _stp_module *m,
     }
     set_fs(oldfs);
 
-    if (rc || (theory != practice)) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
-      _stp_error ("Build-id mismatch [man error::buildid]: \"%s\" byte %d (0x%02x vs 0x%02x) address %#lx rc %d\n",
-		  m->path, j, theory, practice, notes_addr, rc);
+    if (rc == 0) { // got actual data byte
+            hexstring_practice[i++] = hexnibble[practice >> 4];
+            hexstring_practice[i++] = hexnibble[practice & 15];
+    }
+  }
+
+  // have two strings, will travel
+  if (strcmp (hexstring_practice, hexstring_theory)) {
+          _stp_error ("Build-id mismatch [man error::buildid]: \"%s\" address %#lx, expected %s actual %s\n",
+                      m->path, notes_addr, hexstring_theory, hexstring_practice);
       return 1;
-#else
-      /* This branch is a surrogate for kernels affected by Fedora bug
-       * #465873. */
-      _stp_warn (KERN_WARNING
-		 "Build-id mismatch [man error::buildid]: \"%s\" byte %d (0x%02x vs 0x%02x) rc %d\n",
-		 m->path, j, theory, practice, rc);
-#endif
-      break;
-    } /* end mismatch */
-  } /* end per-byte check loop */
+  }
+  
   return 0;
 }
 
@@ -732,7 +746,7 @@ static int _stp_module_check(void)
           dbug_sym(1, "build-id validation [%s]\n", m->name); /* kernel only */
 
           /* skip userspace program */
-          if (m->name[0] != '/') continue;
+          if (m->name[0] == '/') continue;
 
           /* notes end address */
           if (!strcmp(m->name, "kernel")) {
@@ -885,137 +899,147 @@ static int _stp_snprint_addr(char *str, size_t len, unsigned long address,
                                            (int) (flags & _STP_SYM_FILENAME));
   }
 
-  if (name && (flags & _STP_SYM_SYMBOL)) {
-    if ((flags & _STP_SYM_MODULE) && modname && *modname) {
-      if (flags & _STP_SYM_OFFSET) {
-	if (flags & _STP_SYM_SIZE) {
-	  /* symbol, module, offset and size. */
-	  if (flags & _STP_SYM_HEX_SYMBOL)
-	    return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx [%s]%s%s",
-				 prestr, (int64_t) address,
-				 name, offset, size, modname,
-				 exstr, poststr);
-	  else
-	    return _stp_snprintf(str, len, "%s%s+%#lx/%#lx [%s]%s%s",
-				 prestr, name, offset, size,
-				 modname, exstr, poststr);
-	} else {
-	  /* symbol, module, offset. */
-	  if (flags & _STP_SYM_HEX_SYMBOL)
-	    return _stp_snprintf(str, len, "%s%p : %s+%#lx [%s]%s%s",
-				 prestr, (int64_t) address,
-				 name, offset, modname,
-				 exstr, poststr);
-	  else
-	    return _stp_snprintf(str, len, "%s%s+%#lx [%s]%s%s",
-				 prestr, name, offset,
-				 modname, exstr, poststr);
-	}
-      } else {
-	/* symbol plus module */
-	if (flags & _STP_SYM_HEX_SYMBOL)
-	  return _stp_snprintf(str, len, "%s%p : %s [%s]%s%s", prestr,
-			       (int64_t) address, name, modname,
-			       exstr, poststr);
-	else
-	  return _stp_snprintf(str, len, "%s%s [%s]%s%s", prestr, name,
-			       modname, exstr, poststr);
+  switch (flags & ~_STP_SYM_INEXACT) {
+    case _STP_SYM_SYMBOL:
+      if (name) {
+        /* symbol name only */
+        return _stp_snprintf(str, len, "%s%s%s%s", prestr, name, exstr, poststr);
       }
-    } else if (flags & _STP_SYM_OFFSET) {
-      if (flags & _STP_SYM_SIZE) {
-	/* symbol name, offset + size, no module name */
-	if (flags & _STP_SYM_HEX_SYMBOL)
-	  return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx%s%s", prestr,
-			       (int64_t) address, name, offset,
-			       size, exstr, poststr);
-	else
-	  return _stp_snprintf(str, len, "%s%s+%#lx/%#lx%s%s", prestr, name,
-			       offset, size, exstr, poststr);
-      } else {
-	/* symbol name, offset, no module name */
-	if (flags & _STP_SYM_HEX_SYMBOL)
-	  return _stp_snprintf(str, len, "%s%p : %s+%#lx%s%s", prestr,
-			       (int64_t) address, name, offset,
-			       exstr, poststr);
-	else
-	  return _stp_snprintf(str, len, "%s%s+%#lx%s%s", prestr, name,
-			       offset, exstr, poststr);
+      break;
+
+    case _STP_SYM_FILENAME:
+      if (filename) {
+        /* filename */
+        return _stp_snprintf(str, len, "%s%s%s%s", prestr, filename, exstr, poststr);
       }
-    } else {
-      /* symbol name only */
-      if (flags & _STP_SYM_HEX_SYMBOL)
-	return _stp_snprintf(str, len, "%s%p : %s%s%s", prestr,
-			     (int64_t) address, name, exstr, poststr);
-      else
-	return _stp_snprintf(str, len, "%s%s%s%s", prestr, name,
+      break;
+
+    case _STP_SYM_LINENUMBER:
+      if (linenumber) {
+        /* linenumber */
+        return _stp_snprintf(str, len, "%s%#lu%s%s", prestr, linenumber, exstr, poststr);
+      }
+      break;
+
+    case _STP_SYM_FILENAME | _STP_SYM_LINENUMBER:
+      if (filename && linenumber) {
+        /* filename, linenumber */
+        return _stp_snprintf(str, len, "%s%s:%#lu%s%s", prestr, filename, linenumber,
 			     exstr, poststr);
-    }
-  } else {
-    /* no symbol name */
-    if (modname && *modname && (flags & _STP_SYM_MODULE)) {
-      if (flags & _STP_SYM_OFFSET) {
-        if (flags & _STP_SYM_SIZE) {
-          /* hex address, module name, offset + size */
-          return _stp_snprintf(str, len, "%s%p [%s+%#lx/%#lx]%s%s", prestr,
-			       (int64_t) address, modname, offset,
-			       size, exstr, poststr);
-        } else {
-          /* hex address, module name, offset */
-	  return _stp_snprintf(str, len, "%s%p [%s+%#lx]%s%s", prestr,
-			       (int64_t) address, modname, offset,
-			       exstr, poststr);
-        }
-      } else {
-	/* hex address, module name */
-        return _stp_snprintf(str, len, "%s%p [%s]%s%s", prestr,
-			     (int64_t) address, modname, exstr, poststr);
+      } else if (filename) {
+        /* filename, linenumber=?? */
+        return _stp_snprintf(str, len, "%s%s%s%s", prestr, filename, exstr, poststr);
+      } else if (linenumber) {
+        /* filename=??, linenumber */
+        return _stp_snprintf(str, len, "%s??:%#lu%s%s", prestr, linenumber, exstr,
+			     poststr);
       }
-#ifdef STAPCONF_MODULE_TEXT_ADDRESS
-    } if ((flags & _STP_SYM_MODULE) && ! task) {
-      /* No symbol, nor module name, but user really wants one, do one
-	 last try. */
-      struct module *ko;
-      preempt_disable();
-      ko = __module_text_address (address);
-      if (ko && ko->name)
-	{
-	  /* hex address, module name */
-	  int ret = _stp_snprintf(str, len, "%s%p [%s]%s%s", prestr,
-				  (int64_t) address, ko->name, exstr, poststr);
-	  preempt_enable_no_resched();
-	  return ret;
-        }
-      preempt_enable_no_resched();
-      /* no names, hex only */
-      return _stp_snprintf(str, len, "%s%p%s%s", prestr,
-			   (int64_t) address, exstr, poststr);
-#endif
-    } else if ((flags & _STP_SYM_LINENUMBER) && linenumber) {
-        if (flags & _STP_SYM_FILENAME) {
-          if (filename) {
-            /* filename, linenumber */
-            return _stp_snprintf(str, len, "%s%s:%#lu%s%s", prestr,
-                 filename,  linenumber, exstr, poststr);
-          } else {
-            /* filename=??, linenumber */
-            return _stp_snprintf(str, len, "%s??:%#lu%s%s", prestr,
-                 linenumber, exstr, poststr);
-          }
-        } else {
-          /* linenumber */
-          return _stp_snprintf(str, len, "%s%#lu%s%s", prestr,
-               linenumber, exstr, poststr);
-        }
-    } else if ((flags & _STP_SYM_FILENAME) && filename) {
-      /* filename */
-      return _stp_snprintf(str, len, "%s%s%s%s", prestr,
-           filename, exstr, poststr);
-    }else {
-      /* no names, hex only */
-      return _stp_snprintf(str, len, "%s%p%s%s", prestr,
-			   (int64_t) address, exstr, poststr);
-    }
+      break;
+
+    case _STP_SYM_BRIEF:
+      if (name) {
+	/* symbol name and offset */
+        return _stp_snprintf(str, len, "%s%s+%#lx%s%s", prestr, name, offset, exstr,
+			     poststr);
+      }
+      break;
+
+    case _STP_SYM_SIMPLE:
+      if (name && modname) {
+        /* symbol, module, offset. */
+        return _stp_snprintf(str, len, "%s%s+%#lx [%s]%s%s", prestr, name, offset,
+                             modname, exstr, poststr);
+      } else if (name) {
+        /* symbol name, offset, no module name */
+        return _stp_snprintf(str, len, "%s%s+%#lx%s%s", prestr, name, offset, exstr,
+			     poststr);
+      } else if (modname) {
+        /* hex address, module name, offset */
+        return _stp_snprintf(str, len, "%s%p [%s+%#lx]%s%s", prestr, (int64_t) address,
+			     modname, offset, exstr, poststr);
+      }
+      break;
+
+    case _STP_SYM_DATA:
+      if (name && modname) {
+        /* symbol, module, offset and size. */
+        return _stp_snprintf(str, len, "%s%s+%#lx/%#lx [%s]%s%s", prestr, name, offset,
+		             size, modname, exstr, poststr);
+      } else if (name) {
+        /* symbol name, offset + size, no module name */
+        return _stp_snprintf(str, len, "%s%s+%#lx/%#lx%s%s", prestr, name, offset, size,
+		             exstr, poststr);
+      } else if (modname) {
+        /* hex address, module name, offset + size */
+        return _stp_snprintf(str, len, "%s%p [%s+%#lx/%#lx]%s%s", prestr,
+		             (int64_t) address, modname, offset, size, exstr, poststr);
+      }
+      break;
+
+    case _STP_SYM_FULL:
+      if (name && modname) {
+        /* symbol, module, offset and size. */
+        return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx [%s]%s%s", prestr,
+			     (int64_t) address, name, offset, size, modname, exstr,
+			     poststr);
+      } else if (name) {
+        /* symbol name, offset + size, no module name */
+        return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx%s%s", prestr, (int64_t) address,
+			     name, offset, size, exstr, poststr);
+      } else if (modname) {
+        /* hex address, module name, offset + size */
+        return _stp_snprintf(str, len, "%s%p [%s+%#lx/%#lx]%s%s", prestr, (int64_t) address,
+		             modname, offset, size, exstr, poststr);
+      }
+      break;
+
+    case _STP_SYM_FULLER:
+      if (name && modname && filename) {
+        if (linenumber)
+          return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx at %s:%#lu [%s]%s%s",
+                               prestr, (int64_t) address, name, offset, size, filename,
+                               linenumber, modname, exstr, poststr);
+	else
+          return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx at %s [%s]%s%s",
+                               prestr, (int64_t) address, name, offset, size, filename,
+			       modname, exstr, poststr);
+      } else if (name && filename) {
+        if (linenumber)
+          return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx at %s:%#lu%s%s",
+                               prestr, (int64_t) address, name, offset, size, filename,
+                               linenumber, exstr, poststr);
+	else
+          return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx at %s%s%s",
+                               prestr, (int64_t) address, name, offset, size, filename,
+                               exstr, poststr);
+      } else if (modname && filename) {
+        if (linenumber)
+          return _stp_snprintf(str, len, "%s%p [%s+%#lx/%#lx] at %s:%#lu%s%s", prestr,
+                               (int64_t) address, modname, offset, size, filename,
+			       linenumber, exstr, poststr);
+	else
+          return _stp_snprintf(str, len, "%s%p [%s+%#lx/%#lx] at %s%s%s", prestr,
+                               (int64_t) address, modname, offset, size, filename,
+			       exstr, poststr);
+      } else if (name && modname) {
+        /* symbol, module, offset and size. */
+        return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx [%s]%s%s", prestr,
+			     (int64_t) address, name, offset, size, modname,
+                             exstr, poststr);
+      } else if (name) {
+        /* symbol name, offset + size, no module name */
+        return _stp_snprintf(str, len, "%s%p : %s+%#lx/%#lx%s%s", prestr,
+                             (int64_t) address, name, offset, size, exstr, poststr);
+      } else if (modname) {
+        /* hex address, module name, offset + size */
+        return _stp_snprintf(str, len, "%s%p [%s+%#lx/%#lx]%s%s", prestr,
+                             (int64_t) address, modname, offset, size, exstr, poststr);
+      }
+      break;
   }
+
+  /* no names, hex only */
+  return _stp_snprintf(str, len, "%s%p%s%s", prestr, (int64_t) address, exstr, poststr);
 }
 
 static void _stp_print_addr(unsigned long address, int flags,
