@@ -1,7 +1,7 @@
 /* -*- linux-c -*-
  *
  * control channel header
- * Copyright (C) 2009 Red Hat Inc.
+ * Copyright (C) 2009-2018 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -14,9 +14,10 @@
 
 #include <linux/spinlock.h>
 #include <linux/list.h>
+#include "../stp_helper_lock.h"
 
 static struct list_head _stp_ctl_ready_q;
-static spinlock_t _stp_ctl_ready_lock;
+static stp_spinlock_t _stp_ctl_ready_lock;
 static wait_queue_head_t _stp_ctl_wq;
 
 struct _stp_buffer {
