@@ -65,6 +65,7 @@ static atomic_t _stp_transport_state = ATOMIC_INIT(_STP_TS_UNINITIALIZED);
 #include "relay_v2.c"
 #include "debugfs.c"
 #elif STP_TRANSPORT_VERSION == 3
+/* TODO: Ring buffer mechanism is deprecated and could be removed. */
 #include "ring_buffer.c"
 #include "debugfs.c"
 #else
@@ -311,7 +312,7 @@ static void _stp_detach(void)
 }
 
 
-static void _stp_ctl_work_callback(unsigned long val);
+static void _stp_ctl_work_callback(stp_timer_callback_parameter_t unused);
 
 /*
  * Called when stapio opens the control channel.
@@ -320,13 +321,12 @@ static void _stp_attach(void)
 {
 	dbug_trans(1, "attach\n");
 	_stp_pid = current->pid;
-  if (_stp_namespaces_pid < 1)
-    _stp_namespaces_pid = _stp_pid;
+	if (_stp_namespaces_pid < 1)
+		_stp_namespaces_pid = _stp_pid;
 	_stp_transport_data_fs_overwrite(0);
-	init_timer(&_stp_ctl_work_timer);
+
+	timer_setup(&_stp_ctl_work_timer, _stp_ctl_work_callback, 0);
 	_stp_ctl_work_timer.expires = jiffies + STP_CTL_TIMER_INTERVAL;
-	_stp_ctl_work_timer.function = _stp_ctl_work_callback;
-	_stp_ctl_work_timer.data= 0;
 	add_timer(&_stp_ctl_work_timer);
 }
 
@@ -341,7 +341,7 @@ static void _stp_attach(void)
  *	notified. Reschedules itself if someone is still attached
  *	to the cmd channel.
  */
-static void _stp_ctl_work_callback(unsigned long val)
+static void _stp_ctl_work_callback(stp_timer_callback_parameter_t unused)
 {
 	int do_io = 0;
 	unsigned long flags;
@@ -350,10 +350,10 @@ static void _stp_ctl_work_callback(unsigned long val)
 	/* Prevent probe reentrancy while grabbing probe-used locks.  */
 	c = _stp_runtime_entryfn_get_context();
 
-	spin_lock_irqsave(&_stp_ctl_ready_lock, flags);
+	stp_spin_lock_irqsave(&_stp_ctl_ready_lock, flags);
 	if (!list_empty(&_stp_ctl_ready_q))
 		do_io = 1;
-	spin_unlock_irqrestore(&_stp_ctl_ready_lock, flags);
+	stp_spin_unlock_irqrestore(&_stp_ctl_ready_lock, flags);
 
 	_stp_runtime_entryfn_put_context(c);
 
@@ -591,6 +591,8 @@ static struct dentry *_stp_get_root_dir(void)
 #else
 	__stp_root_dir = debugfs_create_dir(name, NULL);
 #endif
+	if (__stp_root_dir == ERR_PTR(-EEXIST)) /* some kernels signal duplication this way */
+	  __stp_root_dir = NULL;
 	if (!__stp_root_dir) {
 		/* Couldn't create it because it is already there, so
 		 * find it. */
