@@ -1,7 +1,7 @@
 /* -*- linux-c -*-
  *
  * dwarf unwinder header file
- * Copyright (C) 2008-2010, 2013 Red Hat Inc.
+ * Copyright (C) 2008-2010, 2013-2019 Red Hat Inc.
  * Copyright (C) 2002-2006 Novell, Inc.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -34,6 +34,8 @@ struct unwind_frame_info
 #include "arm.h"
 #elif defined (__aarch64__)
 #include "arm64.h"
+#elif defined (__mips__)
+#include "mips.h"
 #else
 #error "Unsupported dwarf unwind architecture"
 #endif
@@ -152,12 +154,13 @@ static unsigned long read_ptr_sect(const u8 **pLoc, const void *end,
 				value = _stp_get_unaligned(ptr.p32u++);
 			break;
 		}
+		/* fallthrough */
 	case DW_EH_PE_data8:
 		BUILD_BUG_ON(sizeof(u64) != sizeof(value));
 #else
 		BUILD_BUG_ON(sizeof(u32) != sizeof(value));
 #endif
-	/* fallthrough, see above. */
+		/* fallthrough, see above. */
 	case DW_EH_PE_absptr:
 		if (compat_task)
 		{
@@ -196,9 +199,10 @@ static unsigned long read_ptr_sect(const u8 **pLoc, const void *end,
 	default:
 		return 0;
 	}
-	if ((ptrType & DW_EH_PE_indirect)
-	    && _stp_read_address(value, (unsigned long *)value,
-				 (user ? USER_DS : KERNEL_DS)))
+	if (ptrType & DW_EH_PE_indirect
+	    && _stp_deref_nofault(value, sizeof(value),
+				  (unsigned long *)value, 
+				  (user ? USER_DS : KERNEL_DS)))
 		return 0;
 	*pLoc = ptr.p8;
 
@@ -489,7 +493,7 @@ struct unwind_reg_state {
 	unsigned cfa_is_expr:1;
 };
 
-struct unwind_state {
+struct uw_state {
 	uleb128_t loc;
 	uleb128_t codeAlign;
 	sleb128_t dataAlign;
@@ -500,7 +504,7 @@ struct unwind_state {
 
 struct unwind_context {
     struct unwind_frame_info info;
-    struct unwind_state state;
+    struct uw_state state;
 };
 
 static const struct cfa badCFA = { ARRAY_SIZE(reg_info), 1 };
