@@ -1,24 +1,21 @@
-/* Copyright (C) 2005-2011 Red Hat Inc.
-
-   This file is part of systemtap, and is free software in the public domain.
+/*
+   This file is dedicated to the public domain, pursuant to CC0
+   (https://creativecommons.org/publicdomain/zero/1.0/)
 */
 
 #ifndef _SDT_TYPES_H
 #define _SDT_TYPES_H    1
 
 #define UPROBE1_TYPE 0x31425250 /* "PRB1" (little-endian) */
-#define KPROBE1_TYPE 0x32425250 /* "PRB2" */
 #define UPROBE2_TYPE 0x32425055 /* "UPB2" */
-#define KPROBE2_TYPE 0x3242504b /* "KPB2" */
 #define UPROBE3_TYPE 0x33425055 /* "UPB3" */
 
 typedef enum
   {
     uprobe1_type = UPROBE1_TYPE,
-    kprobe1_type = KPROBE1_TYPE,
     uprobe2_type = UPROBE2_TYPE,
-    kprobe2_type = KPROBE2_TYPE,
-    uprobe3_type = UPROBE3_TYPE
+    uprobe3_type = UPROBE3_TYPE,
+    unknown_probe_type = -1,
   } stap_sdt_probe_type;
 
 typedef struct
