@@ -3,7 +3,7 @@
   the data into a temporary file, calls the systemtap translator and
   then transmits the resulting file back to the client.
 
-  Copyright (C) 2011-2014 Red Hat Inc.
+  Copyright (C) 2011-2018 Red Hat Inc.
 
   This file is part of systemtap, and is free software.  You can
   redistribute it and/or modify it under the terms of the GNU General Public
@@ -73,8 +73,6 @@ static PRStatus spawn_and_wait (const vector<string> &argv, int *result,
                                 const char* fd0, const char* fd1, const char* fd2,
 				const char *pwd, const vector<string>& envVec = vector<string> ());
 
-#define MOK_PUBLIC_CERT_NAME "signing_key.x509"
-#define MOK_PUBLIC_CERT_FILE "/" MOK_PUBLIC_CERT_NAME
 #define MOK_PRIVATE_CERT_NAME "signing_key.priv"
 #define MOK_PRIVATE_CERT_FILE "/" MOK_PRIVATE_CERT_NAME
 #define MOK_CONFIG_FILE "/x509.genkey"
@@ -1127,7 +1125,7 @@ readDataFromSocket(PRFileDesc *sslSocket, const char *requestFileName)
 /* Function:  setupSSLSocket()
  *
  * Purpose:  Configure a socket for SSL.
- *
+ * In case of failure, clean up the incoming tcpSocket and/or any partially setup sslSocket.
  *
  */
 static PRFileDesc * 
@@ -1142,8 +1140,7 @@ setupSSLSocket (PRFileDesc *tcpSocket, CERTCertificate *cert, SECKEYPrivateKey *
   if (sslSocket == NULL)
     {
       server_error (_("Could not import socket into SSL"));
-      nssError ();
-      return NULL;
+      goto error;
     }
    
   /* Set the appropriate flags. */
@@ -1151,32 +1148,28 @@ setupSSLSocket (PRFileDesc *tcpSocket, CERTCertificate *cert, SECKEYPrivateKey *
   if (secStatus != SECSuccess)
     {
       server_error (_("Error setting SSL security for socket"));
-      nssError ();
-      return NULL;
+      goto error;
     }
 
   secStatus = SSL_OptionSet(sslSocket, SSL_HANDSHAKE_AS_SERVER, PR_TRUE);
   if (secStatus != SECSuccess)
     {
       server_error (_("Error setting handshake as server for socket"));
-      nssError ();
-      return NULL;
+      goto error;
     }
 
   secStatus = SSL_OptionSet(sslSocket, SSL_REQUEST_CERTIFICATE, PR_FALSE);
   if (secStatus != SECSuccess)
     {
       server_error (_("Error setting SSL client authentication mode for socket"));
-      nssError ();
-      return NULL;
+      goto error;
     }
 
   secStatus = SSL_OptionSet(sslSocket, SSL_REQUIRE_CERTIFICATE, PR_FALSE);
   if (secStatus != SECSuccess)
     {
       server_error (_("Error setting SSL client authentication mode for socket"));
-      nssError ();
-      return NULL;
+      goto error;
     }
 
   /* Set the appropriate callback routines. */
@@ -1184,18 +1177,16 @@ setupSSLSocket (PRFileDesc *tcpSocket, CERTCertificate *cert, SECKEYPrivateKey *
   secStatus = SSL_AuthCertificateHook (sslSocket, myAuthCertificate, CERT_GetDefaultCertDB());
   if (secStatus != SECSuccess)
     {
-      nssError ();
       server_error (_("Error in SSL_AuthCertificateHook"));
-      return NULL;
+      goto error;
     }
 #endif
 #if 0 /* Use the default */
   secStatus = SSL_BadCertHook(sslSocket, (SSLBadCertHandler)myBadCertHandler, &certErr);
   if (secStatus != SECSuccess)
     {
-      nssError ();
       server_error (_("Error in SSL_BadCertHook"));
-      return NULL;
+      goto error;
     }
 #endif
 #if 0 /* no handshake callback */
@@ -1203,8 +1194,7 @@ setupSSLSocket (PRFileDesc *tcpSocket, CERTCertificate *cert, SECKEYPrivateKey *
   if (secStatus != SECSuccess)
     {
       server_error (_("Error in SSL_HandshakeCallback"));
-      nssError ();
-      return NULL;
+      goto error;
     }
 #endif
 
@@ -1214,11 +1204,27 @@ setupSSLSocket (PRFileDesc *tcpSocket, CERTCertificate *cert, SECKEYPrivateKey *
   if (secStatus != SECSuccess)
     {
       server_error (_("Error configuring SSL server"));
-      nssError ();
-      return NULL;
+      goto error;
     }
 
   return sslSocket;
+
+ error:
+  nssError();
+
+  if (sslSocket) {
+    if (PR_Close (sslSocket) != PR_SUCCESS) {
+      server_error (_("Error closing ssl socket"));
+      nssError ();
+    }
+  } else {
+    if (PR_Close (tcpSocket) != PR_SUCCESS) {
+      server_error (_("Error closing tcp/ssl socket"));
+      nssError ();
+    }
+  }
+  
+  return NULL;
 }
 
 #if 0 /* No client authentication (for now) and not authenticating after each transaction.  */
@@ -1524,15 +1530,22 @@ mok_sign_file (std::string &mok_fingerprint,
 static void
 filter_response_file (const string &file_name, const string &responseDirName)
 {
-  // Filter the server's home directory name
-  string swap = string ("s,") + get_home_directory () + ",<server>,g";
-  vector<string> cmd { "sed", "-i", swap, file_name };
-  (void) stap_system (0, cmd);
+  // Filter the server's home directory name (unless it is "/")
+  string dir = get_home_directory();
+  if (dir != "/")
+    {
+      string swap = string ("s,") + get_home_directory () + ",<server>,g";
+      vector<string> cmd { "sed", "-i", swap, file_name };
+      (void) stap_system (0, cmd);
+    }
 
-  // Filter the server's response directory name
-  swap = string ("s,") + responseDirName + ",<server>,g";
-  cmd = { "sed", "-i", swap, file_name };
-  (void) stap_system (0, cmd);
+  // Filter the server's response directory name (unless it is "/")
+  if (responseDirName != "/")
+    {
+      string swap = string ("s,") + responseDirName + ",<server>,g";
+      vector<string> cmd = { "sed", "-i", swap, file_name };
+      (void) stap_system (0, cmd);
+    }
 }
 
 static privilege_t
@@ -1602,6 +1615,7 @@ generate_mok(string &mok_fingerprint)
   char tmpdir[PATH_MAX] = { '\0' };
   string public_cert_path, private_cert_path, destdir;
   mode_t old_umask;
+  int retlen;
 
   mok_fingerprint.clear ();
 
@@ -1626,7 +1640,14 @@ generate_mok(string &mok_fingerprint)
     }
 
   // Make a temporary directory to store results in.
-  snprintf (tmpdir, PATH_MAX, "%s/stap-server.XXXXXX", mok_path.c_str ());
+  retlen = snprintf (tmpdir, PATH_MAX, "%s/stap-server.XXXXXX", mok_path.c_str ());
+  if (retlen < 0 || retlen >= PATH_MAX)
+    {
+      server_error (_F("Could not create %s name", "temporary directory"));
+      tmpdir[0] = '\0';
+      goto cleanup;
+    }
+
   if (mkdtemp (tmpdir) == NULL)
     {
       server_error (_F("Could not create temporary directory %s: %s", tmpdir, 
@@ -1699,6 +1720,7 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
   unsigned u;
   unsigned i;
   FILE* f;
+  int retlen;
 
   // Save the server version. Do this early, so the client knows what version of the server
   // it is dealing with, even if the request is not fully completed.
@@ -1777,7 +1799,12 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
       struct stat st;
       char *arg;
 
-      snprintf (stapargfile, PATH_MAX, "%s/argv%d", requestDirName.c_str (), i);
+      retlen = snprintf (stapargfile, PATH_MAX, "%s/argv%d", requestDirName.c_str (), i);
+      if (retlen < 0 || retlen >= PATH_MAX)
+        {
+          server_error (_F("Error creating %s name", "path"));
+          return;
+        }
 
       rc = stat(stapargfile, & st);
       if (rc) break;
@@ -1883,12 +1910,23 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
     {
       glob_t globber;
       char pattern[PATH_MAX];
-      snprintf (pattern, PATH_MAX, "%s/*.ko", new_staptmpdir.c_str());
+      int retlen;
+
+      retlen = snprintf (pattern, PATH_MAX, "%s/*.ko", new_staptmpdir.c_str());
+      if (retlen < 0 || retlen >= PATH_MAX)
+        {
+          server_error (_F("Error creating %s name", "pattern"));
+          return;
+        }
+
       rc = glob (pattern, GLOB_ERR, NULL, &globber);
       if (rc)
         server_error (_F("Unable to find a module in %s", new_staptmpdir.c_str()));
       else if (globber.gl_pathc != 1)
-        server_error (_F("Too many modules (%zu) in %s", globber.gl_pathc, new_staptmpdir.c_str()));
+        {
+	  server_error (_F("Too many modules (%zu) in %s", globber.gl_pathc, new_staptmpdir.c_str()));
+	  globfree(&globber);
+	}
       else
         {
 	  if (pr_contains (privilege, pr_stapusr)
@@ -1952,6 +1990,7 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
 	      // resulting module, which wouldn't work.
 	      staprc = 1;
 	  }
+	  globfree(&globber);
         }
     }
 
@@ -2155,6 +2194,7 @@ handle_connection (void *arg)
                         copy for each connection.*/
   vector<string>     argv;
   PRInt32            bytesRead;
+  int		     retlen;
 
   /* Detatch to avoid a memory leak */
   if(max_threads > 0)
@@ -2204,7 +2244,13 @@ handle_connection (void *arg)
 #endif
 
   secStatus = SECFailure;
-  snprintf(tmpdir, PATH_MAX, "%s/stap-server.XXXXXX", getenv("TMPDIR") ?: "/tmp");
+  retlen = snprintf(tmpdir, PATH_MAX, "%s/stap-server.XXXXXX", getenv("TMPDIR") ?: "/tmp");
+  if (retlen < 0 || retlen >= PATH_MAX)
+    {
+      server_error (_F("Error creating %s name", "temporary directory"));
+      tmpdir[0]=0; /* prevent /bin/rm */
+      goto cleanup;
+    }
   rc1 = mkdtemp(tmpdir);
   if (! rc1)
     {
@@ -2214,9 +2260,20 @@ handle_connection (void *arg)
     }
 
   /* Create a temporary files names and directories.  */
-  snprintf (requestFileName, PATH_MAX, "%s/request.zip", tmpdir);
+  retlen = snprintf (requestFileName, PATH_MAX, "%s/request.zip", tmpdir);
+  if (retlen < 0 || retlen >= PATH_MAX)
+    {
+      server_error (_F("Error creating %s name", "request.zip path"));
+      goto cleanup;
+    }
+
+  retlen = snprintf (requestDirName, PATH_MAX, "%s/request", tmpdir);
+  if (retlen < 0 || retlen >= PATH_MAX)
+    {
+      server_error (_F("Error creating %s name", "request directory path"));
+      goto cleanup;
+    }
 
-  snprintf (requestDirName, PATH_MAX, "%s/request", tmpdir);
   rc = mkdir(requestDirName, 0700);
   if (rc)
     {
@@ -2224,7 +2281,13 @@ handle_connection (void *arg)
       goto cleanup;
     }
 
-  snprintf (responseDirName, PATH_MAX, "%s/response", tmpdir);
+  retlen = snprintf (responseDirName, PATH_MAX, "%s/response", tmpdir);
+  if (retlen < 0 || retlen >= PATH_MAX)
+    {
+      server_error (_F("Error creating %s name", "response directory path"));
+      goto cleanup;
+    }
+
   rc = mkdir(responseDirName, 0700);
   if (rc)
     {
@@ -2234,7 +2297,12 @@ handle_connection (void *arg)
   // Set this early, since it gets used for errors to be returned to the client.
   stapstderr = string(responseDirName) + "/stderr";
 
-  snprintf (responseFileName, PATH_MAX, "%s/response.zip", tmpdir);
+  retlen = snprintf (responseFileName, PATH_MAX, "%s/response.zip", tmpdir);
+  if (retlen < 0 || retlen >= PATH_MAX)
+    {
+      server_error (_F("Error creating %s name", "response.zip path"));
+      goto cleanup;
+    }
 
   /* Read data from the socket.
    * If the user is requesting/requiring authentication, authenticate
@@ -2327,6 +2395,9 @@ cleanup:
 	log (_F("Request from [%s]:%d complete", buf, addr.ipv6.port));
     }
 
+  if (privKey)
+    SECKEY_DestroyPrivateKey (privKey); /* Destroy our copy of the private key. */
+  
   /* Increment semephore to indicate this thread is finished. */
   free(t_arg);
   if (max_threads > 0)
@@ -2418,14 +2489,16 @@ accept_connections (PRFileDesc *listenSocket, CERTCertificate *cert)
         fatal(_("No memory available for new thread arg!"));
       t_arg->tcpSocket = tcpSocket;
       t_arg->cert = cert;
-      t_arg->privKey = privKey;
+      t_arg->privKey = SECKEY_CopyPrivateKey(privKey); /* pass by value */
       t_arg->addr = addr;
 
       /* Handle the conncection */
+      int thread_rc = -1;
       if (max_threads > 0)
         /* Create the worker thread and handle the connection. */
-        pthread_create(&tid, NULL, handle_connection, t_arg);
-      else
+        thread_rc = pthread_create(&tid, NULL, handle_connection, t_arg);
+
+      if (thread_rc != 0)
         /* Since max_threads == 0, don't spawn a new thread,
          * just handle in the current thread. */
         handle_connection(t_arg);
@@ -2435,12 +2508,13 @@ accept_connections (PRFileDesc *listenSocket, CERTCertificate *cert)
 				      certUsageSSLServer, NULL/*wincx*/);
       if (secStatus != SECSuccess)
 	{
-	  // Not an error. Exit the loop so a new cert can be generated.
+	  // Not a serious error. Exit the loop so a new cert can be generated.
+          nssError ();
 	  break;
 	}
     }
 
-  SECKEY_DestroyPrivateKey (privKey);
+  SECKEY_DestroyPrivateKey (privKey); /* Safe to destroy, even if thread still running. */
   return SECSuccess;
 }
 
@@ -2455,13 +2529,15 @@ server_main (PRFileDesc *listenSocket)
 {
   int idle_threads;
   int timeout = 0;
+  NSSInitContext *context;
+  SECStatus secStatus;
 
   // Initialize NSS.
-  SECStatus secStatus = nssInit (cert_db_path.c_str ());
-  if (secStatus != SECSuccess)
+  context = nssInitContext (cert_db_path.c_str ());
+  if (context == NULL)
     {
       // Message already issued.
-      return secStatus;
+      return SECFailure;
     }
 
   // Preinitialized here due to jumps to the label 'done'.
@@ -2543,7 +2619,7 @@ server_main (PRFileDesc *listenSocket)
       server_error (_("Unable to shut down server session ID cache"));
       nssError ();
     }
-  nssCleanup (cert_db_path.c_str ());
+  nssCleanup (cert_db_path.c_str (), context);
 
   return secStatus;
 }
@@ -2674,13 +2750,14 @@ listen ()
       if (check_cert (cert_db_path, server_cert_nickname (), use_db_password) != 0)
 	{
 	  // Message already issued
-	  goto done;
+          server_error (_("Cannot check/create certificate, giving up."));
+          goto done;
 	}
 
       // Ensure that our certificate is trusted by our local client.
       // Construct the client database path relative to the server database path.
       SECStatus secStatus = add_client_cert (server_cert_file (),
-					     local_client_cert_db_path ());
+					     local_client_cert_db_path (), db_nssinit);
       if (secStatus != SECSuccess)
 	{
 	  // Not fatal. Other clients may trust the server and trust can be added
