@@ -7,7 +7,7 @@
  * Public License (GPL); either version 2, or (at your option) any
  * later version.
  *
- * Copyright (C) 2005-2010 Red Hat Inc.
+ * Copyright (C) 2005-2018 Red Hat Inc.
  */
 
 #include "staprun.h"
@@ -31,6 +31,53 @@ static int target_pid_failed_p = 0;
    main thread of interruptable events. */
 static pthread_t main_thread;
 
+static void set_nonblocking_std_fds(void)
+{
+  int fd;
+  for (fd = 1; fd < 3; fd++) {
+    /* NB: writing to stderr/stdout blockingly in signal handler is
+     * dangerous since it may prevent the stap process from quitting
+     * gracefully on receiving SIGTERM/etc signals when the stderr/stdout
+     * write buffer is full. PR23891 */
+    int flags = fcntl(fd, F_GETFL);
+    if (flags == -1)
+      continue;
+
+    if (flags & O_NONBLOCK)
+      continue;
+
+    (void) fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+  }
+}
+
+static void set_blocking_std_fds(void)
+{
+  int fd;
+  for (fd = 1; fd < 3; fd++) {
+    /* NB: writing to stderr/stdout blockingly in signal handler is
+     * dangerous since it may prevent the stap process from quitting
+     * gracefully on receiving SIGTERM/etc signals when the stderr/stdout
+     * write buffer is full. PR23891 */
+    int flags = fcntl(fd, F_GETFL);
+    if (flags == -1)
+      continue;
+
+    if (!(flags & O_NONBLOCK))
+      continue;
+
+    (void) fcntl(fd, F_SETFL, flags & ~O_NONBLOCK);
+  }
+}
+
+static void my_exit(int rc)
+{
+  /* to avoid leaving any side-effects on the stdout/stderr devices */
+  if (pending_interrupts > 2)
+    set_blocking_std_fds();
+
+  _exit(rc);
+}
+
 static void *signal_thread(void *arg)
 {
   sigset_t *s = (sigset_t *) arg;
@@ -41,13 +88,19 @@ static void *signal_thread(void *arg)
       _perr("sigwait");
       continue;
     }
-    dbug(2, "sigproc %d (%s)\n", signum, strsignal(signum));
     if (signum == SIGQUIT) {
       load_only = 1; /* flag for stp_main_loop */
       pending_interrupts ++;
-    } else if (signum == SIGINT || signum == SIGHUP || signum == SIGTERM) {
+    } else if (signum == SIGINT || signum == SIGHUP || signum == SIGTERM
+               || signum == SIGPIPE)
+    {
       pending_interrupts ++;
     }
+    if (pending_interrupts > 2) {
+      set_nonblocking_std_fds();
+      pthread_kill (main_thread, SIGURG);
+    }
+    dbug(2, "sigproc %d (%s)\n", signum, strsignal(signum));
   }
   /* Notify main thread (interrupts select). */
   pthread_kill (main_thread, SIGURG);
@@ -153,6 +206,7 @@ static void setup_main_signals(void)
   sigaddset(s, SIGTERM);
   sigaddset(s, SIGHUP);
   sigaddset(s, SIGQUIT);
+  sigaddset(s, SIGPIPE);
   pthread_sigmask(SIG_SETMASK, s, NULL);
   if (pthread_create(&tid, NULL, signal_thread, s) < 0) {
     _perr(_("failed to create thread"));
@@ -238,6 +292,19 @@ void start_cmd(void)
         sh_c_argv[1] = "-c";
         sh_c_argv[2] = target_cmd;
         sh_c_argv[3] = NULL;
+
+        if (read_stdin)
+          {
+            /* close target_cmd's stdin to prevent a data race */
+            char *buf = malloc(sizeof(char) * (strlen(target_cmd) + 13));
+            if (buf == NULL)
+              {
+                 _err (_("Failed to allocate memory.\n"));
+                 _exit(1);
+              }
+            sprintf(buf, "%s < /dev/null", target_cmd);
+            target_cmd = buf;
+          }
       }
     else
       {
@@ -501,6 +568,9 @@ void cleanup_and_exit(int detach, int rc)
   if (monitor)
     monitor_cleanup();
 
+  if (read_stdin)
+    read_stdin_cleanup();
+
   if (exiting)
     return;
   exiting = 1;
@@ -517,10 +587,12 @@ void cleanup_and_exit(int detach, int rc)
   /* OTOH, it may be still be running - but there's no need for
      us to wait for it, considering that the script must have exited
      for another reason.  So, we no longer   while(...wait()...);  here.
-     XXX: we could consider killing it. */
+   */
 
   if (use_old_transport)
     close_oldrelayfs(detach);
+  else if (pending_interrupts > 2)
+    kill_relayfs();
   else
     close_relayfs();
 
@@ -529,7 +601,7 @@ void cleanup_and_exit(int detach, int rc)
 
   if (detach) {
     eprintf(_("\nDisconnecting from systemtap module.\n" "To reconnect, type \"staprun -A %s\"\n"), modname);
-    _exit(0);
+    my_exit(0);
   }
   else if (rename_mod)
     dbug(2, "\nRenamed module to: %s\n", modname);
@@ -552,7 +624,7 @@ void cleanup_and_exit(int detach, int rc)
   pid = fork();
   if (pid < 0) {
           _perr("fork");
-          _exit(-1);
+          my_exit(-1);
   }
 
   if (pid == 0) {			/* child process */
@@ -568,27 +640,27 @@ void cleanup_and_exit(int detach, int rc)
                   execlp("sh", "sh", "-c", cmd, NULL);
                   /* should not return */
                   perror(staprun);
-                  _exit(-1);
+                  my_exit(-1);
           } else {
                   perror("asprintf");
-                  _exit(-1);
+                  my_exit(-1);
           }
   }
 
   /* parent process */
   if (waitpid(pid, &rstatus, 0) < 0) {
           _perr("waitpid");
-          _exit(-1);
+          my_exit(-1);
   }
 
   if (WIFEXITED(rstatus)) {
           if(rc || target_pid_failed_p || rstatus) // if we have an error
-            _exit(1);
+            my_exit(1);
           else
-            _exit(0); //success
+            my_exit(0); //success
   }
 
-  _exit(-1);
+  my_exit(-1);
 }
 
 
@@ -621,7 +693,6 @@ int stp_main_loop(void)
   struct timespec ts;
   struct timespec *timeout = NULL;
   fd_set fds;
-  sigset_t blockset, mainset;
 
 
   setvbuf(ofp, (char *)NULL, _IONBF, 0);
@@ -632,6 +703,7 @@ int stp_main_loop(void)
   if (rc != 0) {
     perror ("Unable to send STP_READY");
     cleanup_and_exit(0, rc);
+    /* NOTREACHED */
   }
 
   flags = fcntl(control_channel, F_GETFL);
@@ -646,19 +718,13 @@ int stp_main_loop(void)
   res = select(control_channel + 1, NULL, NULL, &fds, &tv);
   select_supported = (res == 1 && FD_ISSET(control_channel, &fds));
   dbug(2, "select_supported: %d\n", select_supported);
-  if (select_supported) {
-    /* We block SIGURG to the main thread, except when we call
-       pselect(). This makes sure we won't miss any signals. All other
-       calls are non-blocking, so we defer till pselect() time, which
-       is when we are "sleeping". */
-    sigemptyset(&blockset);
-    sigaddset(&blockset, SIGURG);
-    pthread_sigmask(SIG_BLOCK, &blockset, &mainset);
-  }
 
   if (monitor)
       monitor_setup();
 
+  if (read_stdin)
+      read_stdin_setup();
+
   /* In monitor mode, we must timeout pselect to poll the monitor
      interface. In non-monitor mode, we must timeout pselect so that
      we can handle pending_interrupts. */
@@ -676,12 +742,28 @@ int stp_main_loop(void)
 
     if (pending_interrupts) {
          int btype = STP_EXIT;
-         int rc = write(control_channel, &btype, sizeof(btype));
+         int rc;
+
+	 /* If 'load_only' is set, we don't want to send STP_EXIT,
+	  * which would cause any 'probe end' processing to be
+	  * done. Instead, we'll just detach by calling
+	  * cleanup_and_exit(). This should let the module continue to
+	  * run. */
+	 if (load_only)
+	   {
+	     cleanup_and_exit(load_only /* = detach */, 0);
+	     /* NOTREACHED */
+	   }
+
+         rc = write(control_channel, &btype, sizeof(btype));
          dbug(2, "signal-triggered %d exit rc %d\n", pending_interrupts, rc);
-         if (monitor || (pending_interrupts > 2)) /* user mashing on ^C multiple times */
-                 cleanup_and_exit (load_only /* = detach */, 0);
+         if (monitor || (pending_interrupts > 2))
+	   { /* user mashing on ^C multiple times */
+	     cleanup_and_exit (load_only /* = detach */, 0);
+	     /* NOTREACHED */
+	   }
          else
-                 {} /* await STP_EXIT reply message to kill staprun */
+           {} /* await STP_EXIT reply message to kill staprun */
     }
 
     /* If the runtime does not implement select() on the command
@@ -700,6 +782,7 @@ int stp_main_loop(void)
       if (nb >= 0 || (errno != EINTR && errno != EAGAIN)) {
         _perr(_("Unexpected EOF in read (nb=%ld)"), (long)nb);
         cleanup_and_exit(0, 1);
+	/* NOTREACHED */
       }
 
       if (!select_supported) {
@@ -712,11 +795,12 @@ int stp_main_loop(void)
         // Immediately update screen on input
         if (monitor)
           FD_SET(STDIN_FILENO, &fds);
-	res = pselect(maxfd + 1, &fds, NULL, NULL, timeout, &mainset);
+	res = pselect(maxfd + 1, &fds, NULL, NULL, timeout, NULL);
 	if (res < 0 && errno != EINTR)
 	  {
 	    _perr(_("Unexpected error in select"));
 	    cleanup_and_exit(0, 1);
+	    /* NOTREACHED */
 	  }
       }
       continue;
@@ -731,6 +815,7 @@ int stp_main_loop(void)
       if (write_realtime_data(recvbuf.payload.data, nb)) {
         _perr(_("write error (nb=%ld)"), (long)nb);
         cleanup_and_exit(0, 1);
+	/* NOTREACHED */
       }
       break;
 #endif
@@ -822,8 +907,10 @@ int stp_main_loop(void)
         dbug(2, "got STP_EXIT\n");
         if (monitor)
                 monitor_exited();
-        else
+        else {
                 cleanup_and_exit(0, error_detected);
+		/* NOTREACHED */
+	}
         /* monitor mode exit handled elsewhere, later. */
         break;
       }
@@ -844,6 +931,7 @@ int stp_main_loop(void)
           if (target_cmd)
             kill(target_pid, SIGKILL);
           cleanup_and_exit(0, 1);
+	  /* NOTREACHED */
         } else if (target_cmd) {
           dbug(1, "detaching pid %d\n", target_pid);
 #if WORKAROUND_BZ467568
@@ -859,6 +947,7 @@ int stp_main_loop(void)
               if (target_cmd)
                 kill(target_pid, SIGKILL);
               cleanup_and_exit(0, 1);
+	      /* NOTREACHED */
             }
 #endif
         }
@@ -882,20 +971,24 @@ int stp_main_loop(void)
         struct _stp_msg_start ts;
         struct _stp_msg_ns_pid nspid;
         if (use_old_transport) {
-          if (init_oldrelayfs() < 0)
+	  if (init_oldrelayfs() < 0) {
             cleanup_and_exit(0, 1);
+	    /* NOTREACHED */
+	  }
         } else {
           if (init_relayfs() < 0)
             cleanup_and_exit(0, 1);
+	    /* NOTREACHED */
         }
 
         if (target_namespaces_pid > 0) {
           nspid.target = target_namespaces_pid;
           rc = send_request(STP_NAMESPACES_PID, &nspid, sizeof(nspid));
           if (rc != 0) {
-	          perror ("Unable to send STP_NAMESPACES_PID");
-	          cleanup_and_exit (1, rc);
-	        }
+	    perror ("Unable to send STP_NAMESPACES_PID");
+	    cleanup_and_exit (1, rc);
+	    /* NOTREACHED */
+	  }
         }
 
         ts.target = target_pid;
@@ -903,9 +996,12 @@ int stp_main_loop(void)
 	if (rc != 0) {
 	  perror ("Unable to send STP_START");
 	  cleanup_and_exit(0, rc);
+	  /* NOTREACHED */
 	}
-        if (load_only)
+        if (load_only) {
           cleanup_and_exit(1, 0);
+	  /* NOTREACHED */
+	}
         break;
       }
     default:
