@@ -2,7 +2,7 @@
  *
  * staprun.c - SystemTap module loader
  *
- * Copyright (C) 2005-2018 Red Hat, Inc.
+ * Copyright (C) 2005-2019 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -305,8 +305,10 @@ void disable_kprobes_optimization()
          * 3.8.12 kernel. */
         if (0 && (uname (&uts) == 0) && (strverscmp (uts.release, "3.4") >= 0))
                 return;
-        /* Try again with F24 era kernels. */
-        if ((uname (&uts) == 0) && (strverscmp (uts.release, "4.8") >= 0))
+        /* Disable kprobes optimization due to problems seen on F29 5.0 kernel.
+           PR24416; RCU hang detection with uprobes_onthefly.exp. */
+        /* RHBZ1697531 - x86 kprobe optimization causes rcu hang */
+        if ((0 && uname (&uts) == 0) && (strverscmp (uts.release, "4.8") >= 0))
                 return;
 
         if (getenv ("STAP_PR13193_OVERRIDE"))
