@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2017 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -181,10 +181,15 @@ struct literal_number: public literal
 };
 
 
+struct vardecl;
 struct embedded_expr: public expression
 {
   interned_string code;
+  interned_string code_referents;
+  std::vector<vardecl*> read_referents; /* pragma:read:FOO */
+  std::vector<vardecl*> write_referents; /* pragma:write:BAR */
   bool tagged_p (const char *tag) const;
+  bool tagged_p (const std::string &tag) const;
   bool tagged_p (const interned_string& tag) const;
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -459,6 +464,7 @@ struct functioncall: public expression
   interned_string function;
   std::vector<expression*> args;
   std::vector<functiondecl*> referents;
+  bool synthetic;
   functioncall ();
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -642,7 +648,7 @@ struct vardecl: public symboldecl
   int maxsize; // upperbound on size for arrays
   std::vector<exp_type> index_types; // for arrays only
   literal *init; // for global scalars only
-  bool synthetic; // for probe locals only, don't init on entry
+  bool synthetic; // for probe locals only, don't init on entry; for globals, skip some checking
   bool wrap;
   bool char_ptr_arg; // set in ::emit_common_header(), only used if a formal_arg
 };
@@ -705,7 +711,11 @@ std::ostream& operator << (std::ostream& o, const statement& k);
 struct embeddedcode: public statement
 {
   interned_string code;
+  interned_string code_referents;
+  std::vector<vardecl*> read_referents; /* pragma:read:FOO */
+  std::vector<vardecl*> write_referents; /* pragma:write:BAR */
   bool tagged_p (const char *tag) const;
+  bool tagged_p (const std::string& tag) const;
   bool tagged_p (const interned_string& tag) const;
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -1290,9 +1300,9 @@ struct update_visitor: public visitor
 	    if (this->verbose > 3)
 	      {
 		std::clog << _("replaced ");
-		old_src->print(std::clog);
+		if (old_src) old_src->print(std::clog); else std::clog << "0";
 		std::clog << _(" with ");
-		new_src->print(std::clog);
+		if (new_src) new_src->print(std::clog); else std::clog << "0";
 		std::clog << std::endl;
 	      }
             relaxed_p = false;
