@@ -10,9 +10,13 @@
 #ifndef STRINGTABLE_H
 #define STRINGTABLE_H
 
+#include <functional>
 #include <string>
 #include <cstring>
 
+// TODO use C++17's std::string_view when possible.  It even hashes natively.
+// (some compilers already have std::experimental::string_view)
+
 #if defined(HAVE_BOOST_UTILITY_STRING_REF_HPP)
 #include <boost/version.hpp>
 #include <boost/utility/string_ref.hpp> //header with string_ref
@@ -95,6 +99,11 @@ struct interned_string: public boost::string_ref
   size_t find (const std::string& f) const;
   size_t find (const char *f) const;
 #endif
+
+  size_t find (const interned_string& f) const
+  {
+    return find (static_cast<const boost::string_ref&> (f));
+  }
   
 private:
   static interned_string intern(const std::string& value);
@@ -104,6 +113,26 @@ private:
   // This is private so we can be sure of ownership, from our interned string table.
   interned_string(const boost::string_ref& value): boost::string_ref(value) {}
 };
+
+namespace std {
+  template<> struct hash<interned_string> {
+    size_t operator() (interned_string s) const
+    {
+      // NB: we'd love to be able to hook up to a blob hashing
+      // function in std::hash, but there isn't one.  We don't want
+      // to copy the interned_string into a temporary string just to
+      // hash the thing.
+      //
+      // This code is based on the g++ _Fnv_hash_base ptr/length case.
+      size_t hash = 0;
+      const char* x = s.data();
+      for (size_t i=s.length(); i>0; i--)
+        hash = (hash * 131) + *x++;
+      return hash;
+    }
+  };
+}
+
 #else /* !defined(HAVE_BOOST_UTILITY_STRING_REF_HPP) */
 
 struct interned_string : public std::string {
@@ -132,8 +161,17 @@ private:
   const char* c_str() const;
 };
 
-#endif /* defined(HAVE_BOOST_UTILITY_STRING_REF_HPP) */
+namespace std {
+  template<> struct hash<interned_string> {
+    size_t operator() (interned_string const& s) const
+    {
+      // strings are directly hashable
+      return hash<string>()(s);
+    }
+  };
+}
 
+#endif /* defined(HAVE_BOOST_UTILITY_STRING_REF_HPP) */
 
 #endif // STRINGTABLE_H
 
