@@ -1,5 +1,5 @@
 // tapset for begin/end/error/never
-// Copyright (C) 2005-2011 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 // Copyright (C) 2005-2007 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -65,6 +65,11 @@ struct be_derived_probe: public derived_probe
 
 struct be_derived_probe_group: public generic_dpg<be_derived_probe>
 {
+  friend bool sort_for_bpf(systemtap_session& s,
+			   be_derived_probe_group *be,
+			   std::vector<derived_probe *> &begin_v,
+			   std::vector<derived_probe *> &end_v);
+
 public:
   void emit_module_decls (systemtap_session& s);
   void emit_module_init (systemtap_session& s);
@@ -94,6 +99,8 @@ struct be_builder: public derived_probe_builder
     finished_results.push_back
       (new be_derived_probe(base, location, type, priority));
   }
+
+  virtual string name() { return "begin/end builder"; }
 };
 
 
@@ -184,7 +191,33 @@ be_derived_probe_group::emit_module_exit (systemtap_session& s)
   s.op->newline(-2) << "}";
 }
 
+bool
+sort_for_bpf(systemtap_session& s __attribute__ ((unused)),
+	     be_derived_probe_group *be,
+	     std::vector<derived_probe *> &begin_v,
+	     std::vector<derived_probe *> &end_v)
+{
+  if (!be || be->probes.empty())
+    return false;
 
+  sort(be->probes.begin(), be->probes.end(), be_derived_probe::comp);
+
+  for (auto i = be->probes.begin(); i != be->probes.end(); ++i)
+    {
+      be_derived_probe *p = *i;
+      if (p->type == BEGIN)
+	begin_v.push_back(p);
+    }
+
+  for (auto i = be->probes.begin(); i != be->probes.end(); ++i)
+    {
+      be_derived_probe *p = *i;
+      if (p->type != BEGIN)
+	end_v.push_back(p);
+    }
+
+  return true;
+}
 
 // ------------------------------------------------------------------------
 // never probes are never run
@@ -210,6 +243,8 @@ struct never_builder: public derived_probe_builder
   {
     finished_results.push_back(new never_derived_probe(base, location));
   }
+
+  virtual string name() { return "never builder"; }
 };
 
 
